@@ -7,7 +7,7 @@
 # http://search.cpan.org/dist/Devel-NYTProf/
 #
 ###########################################################
-# $Id: Data.pm 719 2009-03-24 09:21:49Z tim.bunce $
+# $Id$
 ###########################################################
 package Devel::NYTProf::Data;
 
@@ -32,8 +32,13 @@ Access to the data should be via methods in this class to avoid breaking
 encapsulation (and thus breaking your code when the data structures change in
 future versions).
 
-XXX Currently the documentation is out of date as this module is evolving
-rapidly.
+B<NOTE> the documentation is out of date and may not be updated soon.
+It's also likely that the API will change drastically in future.
+It's possible, for example, that the data model will switch to use SQLite
+and the http://search.cpan.org/perldoc?ORLite ORM.
+
+Let me know if you come to depend on a particular API and I'll try to preserve
+it if practical.
 
 =head1 METHODS
 
@@ -43,22 +48,29 @@ rapidly.
 use warnings;
 use strict;
 
-use Carp;
+use Carp qw(carp croak cluck);
 use Cwd qw(getcwd);
 use Scalar::Util qw(blessed);
 
 use Devel::NYTProf::Core;
 use Devel::NYTProf::FileInfo;
 use Devel::NYTProf::SubInfo;
-use Devel::NYTProf::Util qw(make_path_strip_editor strip_prefix_from_paths get_abs_paths_alternation_regex);
+use Devel::NYTProf::Util qw(
+    make_path_strip_editor strip_prefix_from_paths get_abs_paths_alternation_regex
+    trace_level
+);
 
-our $VERSION = '2.09';
+our $VERSION = '4.02';
 
-my $trace = (($ENV{NYTPROF}||'') =~ m/\b trace=(\d+) /x) && $1; # XXX a hack
 
 =head2 new
 
-  $profile = Devel::NYTProf::Data->new( { filename => 'nytprof.out' } );
+  $profile = Devel::NYTProf::Data->new( );
+
+  $profile = Devel::NYTProf::Data->new( {
+    filename => 'nytprof.out', # default
+    quiet    => 0,             # default, 1 to silence message
+  } );
 
 Reads the specified file containing profile data written by L<Devel::NYTProf>,
 aggregates the contents, and returns the results as a blessed data structure.
@@ -68,21 +80,26 @@ aggregates the contents, and returns the results as a blessed data structure.
 
 sub new {
     my $class = shift;
-    my $args = shift || { filename => 'nytprof.out' };
+    my $args = shift || { };
 
-    my $file = $args->{filename}
-        or croak "No filename specified";
+    my $file = $args->{filename} ||= 'nytprof.out';
 
     print "Reading $file\n" unless $args->{quiet};
 
-    my $profile = load_profile_data_from_file($file);
+    my $profile = load_profile_data_from_file(
+        $file,
+        $args->{callback},
+    );
+
+    return undef if $args->{callback};
+
+    print "Processing $file data\n" unless $args->{quiet};
+
     bless $profile => $class;
 
     my $fid_fileinfo = $profile->{fid_fileinfo};
     my $sub_subinfo  = $profile->{sub_subinfo};
 
-    #warn _dumper($profile);
-
     # add profile ref so fidinfo & subinfo objects
     # XXX circular ref, add weaken
     $_ and $_->[7] = $profile for @$fid_fileinfo;
@@ -92,6 +109,30 @@ sub new {
     (my $sub_class = $class) =~ s/\w+$/SubInfo/;
     $_ and bless $_ => $sub_class for values %$sub_subinfo;
 
+    # create profiler_active attribute by subtracting from profiler_duration
+    # currently we only subtract cumulative_overhead_ticks
+    my $attribute = $profile->{attribute};
+    my $overhead_time = $attribute->{cumulative_overhead_ticks} / $attribute->{ticks_per_sec};
+    $attribute->{profiler_active} = $attribute->{profiler_duration} - $overhead_time;
+
+    # find subs that have calls but no fid
+    my @homeless_subs = grep { $_->calls and not $_->fid } values %$sub_subinfo;
+    if (@homeless_subs) { # give them a home...
+        # currently just the first existing fileinfo
+        # XXX ought to create a new dummy fileinfo for them
+        my $new_fi = $profile->fileinfo_of(1);
+        $_->_alter_fileinfo(undef, $new_fi) for @homeless_subs;
+    }
+
+
+    # Where a given eval() has been invoked more than once
+    # rollup the corresponding fids if they're "uninteresting".
+    if (not $args->{skip_collapse_evals}) {
+        for my $fi ($profile->noneval_fileinfos) {
+            $profile->collapse_evals_in($fi);
+        }
+    }
+
     $profile->_clear_caches;
 
     # a hack for testing/debugging
@@ -104,35 +145,142 @@ sub new {
     return $profile;
 }
 
+
+sub collapse_evals_in {
+    my ($profile, $parent_fi) = @_;
+    my $parent_fid = $parent_fi->fid;
+
+    my %evals_on_line;
+    for my $fi ($parent_fi->has_evals) {
+        $profile->collapse_evals_in($fi); # recurse first
+        push @{ $evals_on_line{$fi->eval_line} }, $fi;
+    }
+
+    while ( my ($line, $siblings) = each %evals_on_line) {
+
+        next if @$siblings == 1;
+
+        # compare src code of evals and collapse identical ones
+        my %src_keyed;
+        for my $fi (@$siblings) {
+            my $key = $fi->src_digest;
+            if (!$key) { # include extra info to segregate when there's no src
+                $key .= ',evals' if $fi->has_evals;
+                $key .= ',subs'  if $fi->subs_defined;
+            }
+            push @{$src_keyed{$key}}, $fi;
+        }
+
+        if (trace_level() >= 2) {
+            my @subs  = map { $_->subs_defined } @$siblings;
+            my @evals = map { $_->has_evals(0) } @$siblings;
+            warn sprintf "%d:%d: has %d sibling evals (subs %d, evals %d, keys %d) in %s; fids: %s\n",
+                    $parent_fid, $line, scalar @$siblings, scalar @subs, scalar @evals,
+                    scalar keys %src_keyed,
+                    $parent_fi->filename,
+                    join(" ", map { $_->fid } @$siblings);
+            if (trace_level() >= 2) {
+                for my $si (@subs) {
+                    warn sprintf "%d:%d evals: define sub %s in fid %s\n",
+                            $parent_fid, $line, $si->subname, $si->fid;
+                }
+                for my $fi (@evals) {
+                    warn sprintf "%d:%d evals: execute eval %s\n",
+                            $parent_fid, $line, $fi->filename;
+                }
+            }
+        }
+
+        # if 'too many' distinct eval source keys then simply collapse all
+        my $max_evals_siblings = $ENV{NYTPROF_MAX_EVAL_SIBLINGS} || 200;
+        if (values %src_keyed > $max_evals_siblings) {
+            $parent_fi->collapse_sibling_evals(@$siblings);
+        }
+        else {
+            # finesse: consider each distinct src in turn
+
+            while ( my ($key, $src_same_fis) = each %src_keyed ) {
+                next if @$src_same_fis == 1; # unique src key
+                my @fids = map { $_->fid } @$src_same_fis;
+
+                if (grep { $_->has_evals(0) } @$src_same_fis) {
+                    warn "evals($key): collapsing skipped due to evals in @fids\n" if trace_level() >= 3;
+                }
+                else {
+                    warn "evals($key): collapsing identical: @fids\n" if trace_level() >= 3;
+                    my $fi = $parent_fi->collapse_sibling_evals(@$src_same_fis);
+                    @$src_same_fis = ( $fi ); # update list in-place
+                }
+            }
+        }
+    }
+}
+
 sub _caches       { return shift->{caches} ||= {} }
 sub _clear_caches { return delete shift->{caches} }
 
+sub attributes {
+    return shift->{attribute} || {};
+}
+
+sub options {
+    return shift->{option} || {};
+}
+
 sub subname_subinfo_map {
     return { %{ shift->{sub_subinfo} } }; # shallow copy
 }
 
-# default:
-# { pkgname => [ subinfo1, subinfo2, ... ], ... }
-# merged:
-# { pkgname => [ single_merged_subinfo ], ...  }
+sub _disconnect_subinfo {
+    my ($self, $si) = @_;
+    my $subname = $si->subname;
+    my $si2 = delete $self->{sub_subinfo}{$subname};
+    # sanity check
+    carp sprintf "disconnect_subinfo: deleted entry %s %s doesn't match argument %s %s",
+            ($si2) ? ($si2, $si2->subname) : ('undef', 'undef'),
+            $si, $subname
+        if $si2 != $si or $si2->subname ne $subname;
+    # do more?
+}
+
+
+# package_tree_subinfo_map is like package_subinfo_map but returns
+# nested data instead of flattened.
+# for "Foo::Bar::Baz" package:
+# { Foo => { '' => [...], '::Bar' => { ''=>[...], '::Baz'=>[...] } } }
+# if merged is true then array contains a single 'merged' subinfo
 sub package_subinfo_map {
     my $self = shift;
-    my ($merged) = @_;
+    my ($merge_subs, $nested_pkgs) = @_;
+
+    my %pkg;
+    my %to_merge;
 
     my $all_subs = $self->subname_subinfo_map;
-    my %pkg;
     while ( my ($name, $subinfo) = each %$all_subs ) {
         $name =~ s/^(.*::).*/$1/; # XXX $subinfo->package
-        push @{ $pkg{$name} }, $subinfo;
-    }
-    if ($merged) {
-        while ( my ($pkg_name, $subinfos) = each %pkg ) {
-            my $subinfo = shift(@$subinfos)->clone;
-            $subinfo->merge_in($_) for @$subinfos;
-            # replace the many with the one
-            @$subinfos = ($subinfo);
+        my $subinfos;
+        if ($nested_pkgs) {
+            my @parts = split /::/, $name;
+            my $node = $pkg{ shift @parts } ||= {};
+            $node = $node->{ shift @parts } ||= {} while @parts;
+            $subinfos = $node->{''} ||= [];
         }
+        else {
+            $subinfos = $pkg{$name} ||= [];
+        }
+        push @$subinfos, $subinfo;
+        $to_merge{$subinfos} = $subinfos if $merge_subs;
     }
+
+    for my $subinfos (values %to_merge) {
+        my $subinfo = shift(@$subinfos)->clone;
+        $subinfo->merge_in($_, src_keep => 1)
+            for @$subinfos;
+        # replace the many with the one
+        @$subinfos = ($subinfo);
+    }
+
     return \%pkg;
 }
 
@@ -187,15 +335,24 @@ sub packages_at_depth_subinfo {
 sub all_fileinfos {
     my @all = @{shift->{fid_fileinfo}};
     shift @all;    # drop fid 0
-    return @all;
+    # return all non-nullified fileinfos
+    return grep { $_->fid } @all;
+}
+
+sub eval_fileinfos {
+    return grep {  $_->eval_line } shift->all_fileinfos;
+}
+
+sub noneval_fileinfos {
+    return grep { !$_->eval_line } shift->all_fileinfos;
 }
 
 
 sub fileinfo_of {
-    my $self = shift;
-    my $arg  = shift;
+    my ($self, $arg, $silent_if_undef) = @_;
+
     if (not defined $arg) {
-        carp "Can't resolve fid of undef value";
+        carp "Can't resolve fid of undef value" unless $silent_if_undef;
         return undef;
     }
 
@@ -208,34 +365,24 @@ sub fileinfo_of {
         return undef;
     }
 
-    return $self->{fid_fileinfo}[$fid];
+    my $fi = $self->{fid_fileinfo}[$fid];
+    return undef unless defined $fi->fid; # nullified?
+    return $fi;
 }
 
 
-# map of { eval_fid => base_fid, ... }
-sub eval_fid_2_base_fid_map {
-    my ($self, $flatten_evals) = @_;
-    $flatten_evals ||= 0;
+sub subinfo_of {
+    my ($self, $subname) = @_;
 
-    my $caches = $self->_caches;
-    my $cache_key = "eval_fid_2_base_fid_map:$flatten_evals";
-    return $caches->{$cache_key} if $caches->{$cache_key};
-
-    my $fid_fileinfo = $self->{fid_fileinfo} || [];
-    my $eval_fid_map = {};
-
-    for my $fi (@$fid_fileinfo) {
-        my $base_fi = $fi && $fi->eval_fi
-            or next;
-
-        while ($flatten_evals and my $b_eval_fi = $base_fi->eval_fi) {
-            $base_fi = $b_eval_fi;
-        }
-        $eval_fid_map->{ $fi->fid } = $base_fi->fid;
+    if (not defined $subname) {
+        cluck "Can't resolve subinfo of undef value";
+        return undef;
     }
 
-    $caches->{$cache_key} = $eval_fid_map;
-    return $eval_fid_map;
+    my $si = $self->{sub_subinfo}{$subname}
+        or cluck "Can't resolve subinfo of '$subname'";
+
+    return $si;
 }
 
 
@@ -277,47 +424,49 @@ sub dump_profile_data {
     my $separator  = $args->{separator} || '';
     my $filehandle = $args->{filehandle} || \*STDOUT;
 
-    #skip_stdlib
-
     # shallow clone and add sub_caller for migration of tests
     my $startnode = $self;
 
     $self->_clear_caches;
 
-    my $abs_path_regex = $^O eq "MSWin32" ? qr,^\w:/, : qr,^/,;
-    my @abs_inc = grep { $_ =~ $abs_path_regex } $self->inc;
-    my $is_lib_regex = get_abs_paths_alternation_regex(\@abs_inc);
-
     my $callback = sub {
         my ($path, $value) = @_;
 
-        if ($args->{skip_stdlib}) {
+        # not needed currently
+        #if ($path->[0] eq 'attribute' && @$path == 1) { my %v = %$value; return ({}, \%v); }
 
-            # for fid_fileinfo don't dump internal details of lib modules
+        if (my $hook = $args->{skip_fileinfo_hook}) {
+
+            # for fid_fileinfo elements...
             if ($path->[0] eq 'fid_fileinfo' && @$path==2) {
-                my $is_lib = ($value->filename =~ $is_lib_regex) ? 1 : 0;
-                return { skip_internal_details => $is_lib };
+                my $fi = $value;
+
+                # skip nullified fileinfo
+                return undef unless $fi->fid;
+
+                # don't dump internal details of lib modules
+                return ({ skip_internal_details => scalar $hook->($fi, $path, $value) }, $value);
             }
 
             # skip sub_subinfo data for 'library modules'
             if ($path->[0] eq 'sub_subinfo' && @$path==2 && $value->[0]) {
                 my $fi = $self->fileinfo_of($value->[0]);
-                return undef if $fi->filename =~ $is_lib_regex;
+                return undef if !$fi or $hook->($fi, $path, $value);
             }
 
             # skip fid_*_time data for 'library modules'
             if ($path->[0] =~ /^fid_\w+_time$/ && @$path==2) {
                 my $fi = $self->fileinfo_of($path->[1]);
-                return undef if $fi->filename =~ $is_lib_regex
-                         or $fi->filename =~ m!^/\.\.\./!;
+                return undef if !$fi or $hook->($fi, $path, $value);
             }
         }
-        return {};
+        return ({}, $value);
     };
 
     _dump_elements($startnode, $separator, $filehandle, [], $callback);
 }
 
+
 sub _dump_elements {
     my ($r, $separator, $fh, $path, $callback) = @_;
     my $pad = "    ";
@@ -348,11 +497,13 @@ sub _dump_elements {
         my $value = ($is_hash) ? $r->{$key} : $r->[$key];
 
         # skip undef elements in array
-        next if !defined($value) && !$is_hash;
+        next if !$is_hash && !defined($value);
+        # skip refs to empty arrays in array
+        next if !$is_hash && ref $value eq 'ARRAY' && !@$value;
 
         my $dump_opts = {};
         if ($callback) {
-            $dump_opts = $callback->([ @$path, $key ], $value);
+            ($dump_opts, $value) = $callback->([ @$path, $key ], $value);
             next if not $dump_opts;
         }
 
@@ -407,8 +558,8 @@ sub get_fid_line_data {
   $profile->normalize_variables;
 
 Traverses the profile data structure and normalizes highly variable data, such
-as the time, in order that the data can more easily be compared. This is used,
-for example, by the test suite.
+as the time, in order that the data can more easily be compared. This is mainly of
+use to the test suite.
 
 The data normalized is:
 
@@ -440,21 +591,27 @@ filenames: eval sequence numbers, like "(re_eval 2)" are changed to 0
 
 
 sub normalize_variables {
-    my $self       = shift;
+    my ($self, $normalize_options) = @_;
+
+    if ($normalize_options) {
+        %{ $self->options } = ();
+    }
+
+    my $attributes = $self->attributes;
 
     for my $attr (qw(
         basetime xs_version perl_version clock_id ticks_per_sec nv_size
         profiler_duration profiler_end_time profiler_start_time
+        cumulative_overhead_ticks profiler_active
         total_stmts_duration total_stmts_measured total_stmts_discounted
-        total_sub_calls
+        total_sub_calls sawampersand_line
     )) {
-        $self->{attribute}{$attr} = 0;
+        $attributes->{$attr} = 0 if exists $attributes->{$attr};
     }
 
-    my $abs_path_regex = $^O eq "MSWin32" ? qr,^\w:/, : qr,^/,;
-    my @abs_inc = grep { $_ =~ $abs_path_regex } $self->inc;
-    my $is_lib_regex = get_abs_paths_alternation_regex(\@abs_inc);
-
+    for my $attr (qw(PL_perldb cumulative_overhead_ticks)) {
+        delete $attributes->{$attr};
+    }
 
     # normalize line data
     for my $level (qw(line block sub)) {
@@ -466,8 +623,18 @@ sub normalize_variables {
         }
     }
 
-    # zero sub into and sub caller times
-    $_->normalize_for_test for values %{ $self->{sub_subinfo} };
+    my $sub_subinfo = $self->{sub_subinfo};
+    for my $subname (keys %$sub_subinfo) {
+        my $si = $self->{sub_subinfo}{$subname};
+        # zero sub info and sub caller times etc.
+        my $newname = $si->normalize_for_test;
+        if ($newname ne $subname) {
+            warn "Normalizing $subname to $newname overwrote other data\n"
+                if $sub_subinfo->{$newname};
+            $sub_subinfo->{$newname} = delete $sub_subinfo->{$subname};
+        }
+    }
+
     $_->normalize_for_test for $self->all_fileinfos;
 
     return;
@@ -493,12 +660,8 @@ sub _filename_to_fid {
     my $self = shift;
     my $caches = $self->_caches;
     return $caches->{_filename_to_fid_cache} ||= do {
-        my $fid_fileinfo = $self->{fid_fileinfo} || [];
         my $filename_to_fid = {};
-        for my $fid (1 .. @$fid_fileinfo - 1) {
-            my $filename = $fid_fileinfo->[$fid][0];
-            $filename_to_fid->{$filename} = $fid;
-        }
+        $filename_to_fid->{$_->filename} = $_->fid for $self->all_fileinfos;
         $filename_to_fid;
     };
 }
@@ -527,89 +690,41 @@ subroutines defined on that line, typically just one.
 
 sub subs_defined_in_file {
     my ($self, $fid, $incl_lines) = @_;
-    $fid = $self->resolve_fid($fid);
-    $incl_lines ||= 0;
-    $incl_lines = 0 if $fid == 0;
-    my $caches = $self->_caches;
-
-    my $cache_key = "subs_defined_in_file:$fid:$incl_lines";
-    return $caches->{$cache_key} if $caches->{$cache_key};
+    croak "incl_lines is deprecated in subs_defined_in_file, use subs_defined_in_file_by_line instead" if $incl_lines;
 
     my $fi = $self->fileinfo_of($fid)
         or return;
-    my %subs = %{ $fi->subs || {} }; # shallow copy
 
-    if ($incl_lines) {    # add in the first-line-number keys
-        croak "Can't include line numbers without a fid" unless $fid;
-        for (values %subs) {
-            next unless defined(my $first_line = $_->first_line);
-            push @{$subs{$first_line}}, $_;
-        }
-    }
+    $fid = $fi->fid;
+    my $caches = $self->_caches;
+
+    my $cache_key = "subs_defined_in_file:$fid";
+    return $caches->{$cache_key} if $caches->{$cache_key};
+
+    my %subs = map { $_->subname => $_ } $fi->subs_defined;
 
     $caches->{$cache_key} = \%subs;
     return $caches->{$cache_key};
 }
 
 
-=head2 subname_at_file_line
-
-  @subname = $profile->subname_at_file_line($file, $line_number);
-  $subname = $profile->subname_at_file_line($file, $line_number);
-
-=cut
-
-
-sub subname_at_file_line {
-    my ($self, $fid, $line) = @_;
-
-    my $subs = $self->subs_defined_in_file($fid, 0);
-
-    # XXX could be done more efficiently
-    my @subname;
-    for my $sub_info (values %$subs) {
-        next
-            if $sub_info->first_line > $line
-            or $sub_info->last_line < $line;
-        push @subname, $sub_info->subname;
+sub subs_defined_in_file_by_line {
+    my $subs = shift->subs_defined_in_file(@_);
+    my %line2subs;
+    for (values %$subs) {
+        my $first_line = $_->first_line || 0; # 0 = xsub?
+        push @{$line2subs{$first_line}}, $_;
     }
-    @subname = sort { length($a) <=> length($b) } @subname;
-    return @subname if wantarray;
-    carp
-        "Multiple subs at $fid line $line (@subname) but subname_at_file_line called in scalar context"
-        if @subname > 1;
-    return $subname[0];
-}
-
-
-sub fid_filename {
-    my ($self, $fid) = @_;
-
-    my $fileinfo = $self->{fid_fileinfo}->[$fid]
-        or return undef;
-
-    while ($fileinfo->[1]) {    # is an eval
-
-        # eg string eval
-        # eg [ "(eval 6)[/usr/local/perl58-i/lib/5.8.6/Benchmark.pm:634]", 2, 634 ]
-        warn sprintf "fid_filename: fid %d -> %d for %s\n", $fid, $fileinfo->[1], $fileinfo->[0]
-            if $trace;
-
-        # follow next link in chain
-        my $outer_fid = $fileinfo->[1];
-        $fileinfo = $self->{fid_fileinfo}->[$outer_fid];
-    }
-
-    return $fileinfo->[0];
+    return \%line2subs;
 }
 
 
 =head2 file_line_range_of_sub
 
-  ($file, $fid, $first, $last) = $profile->file_line_range_of_sub("main::foo");
+  ($file, $fid, $first, $last, $fi) = $profile->file_line_range_of_sub("main::foo");
 
-Returns the filename, fid, and first and last line numbers for the specified
-subroutine (which must be fully qualified with a package name).
+Returns the filename, fid, and first and last line numbers, and fileinfo object
+for the specified subroutine (which must be fully qualified with a package name).
 
 Returns an empty list if the subroutine name is not in the profile data.
 
@@ -617,12 +732,11 @@ The $fid return is the 'original' fid associated with the file the subroutine wa
 
 The $file returned is the source file that defined the subroutine.
 
-Where is a subroutine is defined within a string eval, for example, the fid
-will be the pseudo-fid for the eval, and the $file will be the filename that
-executed the eval.
+Subroutines that are implemented in XS have a line range of 0,0 and a possibly
+unknown file (if NYTProf couldn't find a good match based on the package name).
 
-Subroutines that are implemented in XS have a line range of 0,0 and currently
-don't have an associated file.
+Subroutines that were called but only returned via an exception may have a line
+range of undef,undef if they're xsubs or were defined before NYTProf was enabled.
 
 =cut
 
@@ -630,25 +744,16 @@ don't have an associated file.
 sub file_line_range_of_sub {
     my ($self, $sub) = @_;
 
-    my $sub_subinfo = $self->{sub_subinfo}{$sub}
+    my $sub_subinfo = $self->subinfo_of($sub)
         or return;    # no such sub
     my ($fid, $first, $last) = @$sub_subinfo;
 
-    my $fileinfo = $fid && $self->{fid_fileinfo}->[$fid];
-    while ($fileinfo->[1]) {    # file is an eval
+    return if not $fid; # sub has no known file
 
-        # eg string eval
-        # eg [ "(eval 6)[/usr/local/perl58-i/lib/5.8.6/Benchmark.pm:634]", 2, 634 ]
-        warn sprintf "file_line_range_of_sub: %s: fid %d -> %d for %s\n", $sub, $fid, $fileinfo->[1], $fileinfo->[0]
-            if $trace;
-        $first = $last = $fileinfo->[2] if 1;    # XXX control via param?
+    my $fileinfo = $fid && $self->fileinfo_of($fid)
+        or croak "No fid_fileinfo for sub $sub fid '$fid'";
 
-        # follow next link in chain
-        my $outer_fid = $fileinfo->[1];
-        $fileinfo = $self->{fid_fileinfo}->[$outer_fid];
-    }
-
-    return ($fileinfo->[0], $fid, $first, $last);
+    return ($fileinfo->filename, $fid, $first, $last, $fileinfo);
 }
 
 
@@ -701,77 +806,6 @@ sub resolve_fid {
 }
 
 
-=head2 line_calls_for_file
-
-  $line_calls_hash = $profile->line_calls_for_file( $file );
-
-Returns a reference to a hash containing information about subroutine calls
-made at individual lines within a source file. The $file
-argument can be an integer file id (fid) or a file path. Returns undef if
-no subroutine calling information is available.
-
-The keys of the returned hash are line numbers. The values are references to
-hashes with fully qualified subroutine names as keys. Each hash value is an
-reference to an array containing an integer call count (how many times the sub
-was called from that line of that file) and an inclusive time (how much time
-was spent inside the sub when it was called from that line of that file).
-
-For example, if the following was line 42 of a file C<foo.pl>:
-
-  ++$wiggle if foo(24) == bar(42);
-
-that line was executed once, and foo and bar were imported from pkg1, then
-$profile->line_calls_for_file( 'foo.pl' ) would return something like:
-
-  {
-      42 => {
-	  'pkg1::foo' => [ 1, 0.02093 ],
-	  'pkg1::bar' => [ 1, 0.00154 ],
-      },
-  }
-
-=cut
-
-sub line_calls_for_file {
-    my ($self, $fid, $include_evals) = @_;
-    my $orig_fi = $self->fileinfo_of($fid);
-
-    # shallow copy
-    my $line_calls = { %{ $orig_fi->sub_call_lines } };
-    return $line_calls unless $include_evals;
-
-    for my $fi (@{ $orig_fi->has_evals(1) || [] }) {
-        # { line => { subname => [...] }, ... }
-        my $sub_call_lines = $fi->sub_call_lines;
-
-        # $outer_line is the line of the eval
-        # XXX outer(1) is a little inefficient, could refactor the loop to
-        # separate top-level evals from nested evals and use the outer_line
-        # from the top level evals
-        my (undef, $outer_line) = $fi->outer(1); # outermost
-
-        while (my ($line, $sub_calls_hash) = each %$sub_call_lines) {
-
-            my $ci_for_subs = $line_calls->{$outer_line || $line} ||= {};
-
-            while (my ($subname, $callinfo) = each %$sub_calls_hash) {
-
-                my $ci = $ci_for_subs->{$subname} ||= [];
-                if (!@$ci) {    # typical case
-                    @$ci = @$callinfo;
-                }
-                else {          # e.g., multiple calls inside the same string eval
-                    #warn "merging calls to $subname from fid $caller_fid line $caller_line ($outer_line || $line)";
-                    $ci->[$_] += $callinfo->[$_] for 0..5;
-                    $ci->[6]   = $callinfo->[6] if $callinfo->[6] > $ci->[6]; # NYTP_SCi_REC_DEPTH
-                }
-            }
-        }
-    }
-    return $line_calls;
-}
-
-
 sub package_fids {
     my ($self, $package) = @_;
     my @fids;
@@ -798,6 +832,10 @@ __END__
 
 XXX
 
+=head1 LIMITATION
+
+There's currently no way to merge profile data from multiple files.
+
 =head1 SEE ALSO
 
 L<Devel::NYTProf>
@@ -811,7 +849,7 @@ B<Steve Peters>, C<< <steve at fisharerojo.org> >>
 =head1 COPYRIGHT AND LICENSE
 
  Copyright (C) 2008 by Adam Kaplan and The New York Times Company.
- Copyright (C) 2008 by Tim Bunce, Ireland.
+ Copyright (C) 2008,2009 by Tim Bunce, Ireland.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
